#ifndef MSDOS_H 
#define MSDOS_H 
int  ffsl(unsigned long  x);
void  FreeFontRecord(struct  _FontPathRec *pFP);
int  SetDefaultFontPath(char  *name);
struct  _FontPathRec *GetFontPath(void);
int  FontFileRead(char  *buf,unsigned int  itemsize,unsigned int  nitems,unsigned char  *fid);
int  FontFileSkip(unsigned int  bytes,unsigned char  *fid);
int  FontFilePropLoad(char  *fontname,unsigned int  length,struct  _Font * *font,struct  _FontInfo *fi,struct  _DIXFontProp * *props);
void  FontUnload(struct  _Font *font);
void  OsInit(void);
void OsInitColors(void);
void OsCloseColors(void);
void  AutoResetServer(void);
void  GiveUp(int);
void  Error(char  *str);
unsigned long  GetTimeInMillis(void);
void  UseMsg(void);
void  ProcessCommandLine(int  argc,char  * *argv);
void  *Xalloc(unsigned int  amount);
void  *Xcalloc(unsigned int  amount);
void  *Xrealloc(void  *ptr,unsigned int  amount);
void  Xfree(void  *ptr);
void  FatalError(char  *f,...);
void  ErrorF(char  *f,...);
void  stack_dump(char *bp);
void XdmcpUseMsg(void);
int XdmcpOptions(int argc, char **argv, int i);
void client_init(char *);
#endif 
